@@ -15,6 +15,7 @@
 # limitations under the License.
 
 # pylint: disable=too-many-public-methods
+# pylint: disable=too-many-lines
 
 """MinIO Admin Client to perform MinIO administration operations."""
 
@@ -24,7 +25,7 @@ import json
 import os
 from datetime import timedelta
 from enum import Enum, unique
-from typing import Any, TextIO, Tuple, cast
+from typing import Any, Optional, TextIO, Tuple, cast
 from urllib.parse import urlunsplit
 
 import certifi
@@ -116,12 +117,13 @@ class MinioAdmin:
 
     def __init__(
             self,
+            *,
             endpoint: str,
             credentials: Provider,
             region: str = "",
             secure: bool = True,
             cert_check: bool = True,
-            http_client: PoolManager | None = None,
+            http_client: Optional[PoolManager] = None,
     ):
         url = _parse_url(("https://" if secure else "http://") + endpoint)
         if not isinstance(credentials, Provider):
@@ -155,23 +157,24 @@ class MinioAdmin:
         self._cert_check = cert_check
         self._http = http_client
         self._user_agent = _DEFAULT_USER_AGENT
-        self._trace_stream: TextIO | None = None
+        self._trace_stream: Optional[TextIO] = None
 
     def __del__(self):
         self._http.clear()
 
     def _url_open(
             self,
+            *,
             method: str,
             command: _COMMAND,
-            query_params: DictType | None = None,
-            body: bytes | None = None,
+            query_params: Optional[DictType] = None,
+            body: Optional[bytes] = None,
             preload_content: bool = True,
     ) -> BaseHTTPResponse:
         """Execute HTTP request."""
         creds = self._provider.retrieve()
 
-        url = url_replace(self._url, path="/minio/admin/v3/"+command.value)
+        url = url_replace(url=self._url, path="/minio/admin/v3/"+command.value)
         query = []
         for key, values in sorted((query_params or {}).items()):
             values = values if isinstance(values, (list, tuple)) else [values]
@@ -179,7 +182,7 @@ class MinioAdmin:
                 f"{queryencode(key)}={queryencode(value)}"
                 for value in sorted(values)
             ]
-        url = url_replace(url, query="&".join(query))
+        url = url_replace(url=url, query="&".join(query))
 
         content_sha256 = sha256_hash(body)
         date = time.utcnow()
@@ -196,13 +199,13 @@ class MinioAdmin:
             headers["Content-Length"] = str(len(body))
 
         headers = sign_v4_s3(
-            method,
-            url,
-            self._region,
-            headers,
-            creds,
-            content_sha256,
-            date,
+            method=method,
+            url=url,
+            region=self._region,
+            headers=headers,
+            credentials=creds,
+            content_sha256=content_sha256,
+            date=date,
         )
 
         if self._trace_stream:
@@ -291,8 +294,8 @@ class MinioAdmin:
     def service_restart(self) -> str:
         """Restart MinIO service."""
         response = self._url_open(
-            "POST",
-            _COMMAND.SERVICE,
+            method="POST",
+            command=_COMMAND.SERVICE,
             query_params={"action": "restart"}
         )
         return response.data.decode()
@@ -300,8 +303,8 @@ class MinioAdmin:
     def service_stop(self) -> str:
         """Stop MinIO service."""
         response = self._url_open(
-            "POST",
-            _COMMAND.SERVICE,
+            method="POST",
+            command=_COMMAND.SERVICE,
             query_params={"action": "stop"}
         )
         return response.data.decode()
@@ -309,8 +312,8 @@ class MinioAdmin:
     def update(self) -> str:
         """Update MinIO."""
         response = self._url_open(
-            "POST",
-            _COMMAND.UPDATE,
+            method="POST",
+            command=_COMMAND.UPDATE,
             query_params={"updateURL": ""}
         )
         return response.data.decode()
@@ -318,16 +321,16 @@ class MinioAdmin:
     def info(self) -> str:
         """Get MinIO server information."""
         response = self._url_open(
-            "GET",
-            _COMMAND.INFO,
+            method="GET",
+            command=_COMMAND.INFO,
         )
         return response.data.decode()
 
     def account_info(self, prefix_usage: bool = False) -> str:
         """Get usage information for the authenticating account"""
         response = self._url_open(
-            "GET",
-            _COMMAND.ACCOUNT_INFO,
+            method="GET",
+            command=_COMMAND.ACCOUNT_INFO,
             query_params={"prefix-usage": "true"} if prefix_usage else None,
         )
         return response.data.decode()
@@ -337,8 +340,8 @@ class MinioAdmin:
         body = json.dumps(
             {"status": "enabled", "secretKey": secret_key}).encode()
         response = self._url_open(
-            "PUT",
-            _COMMAND.ADD_USER,
+            method="PUT",
+            command=_COMMAND.ADD_USER,
             query_params={"accessKey": access_key},
             body=encrypt(body, self._provider.retrieve().secret_key),
         )
@@ -347,8 +350,8 @@ class MinioAdmin:
     def user_disable(self, access_key: str) -> str:
         """Disable user."""
         response = self._url_open(
-            "PUT",
-            _COMMAND.SET_USER_STATUS,
+            method="PUT",
+            command=_COMMAND.SET_USER_STATUS,
             query_params={"accessKey": access_key, "status": "disabled"}
         )
         return response.data.decode()
@@ -356,8 +359,8 @@ class MinioAdmin:
     def user_enable(self, access_key: str) -> str:
         """Enable user."""
         response = self._url_open(
-            "PUT",
-            _COMMAND.SET_USER_STATUS,
+            method="PUT",
+            command=_COMMAND.SET_USER_STATUS,
             query_params={"accessKey": access_key, "status": "enabled"}
         )
         return response.data.decode()
@@ -365,8 +368,8 @@ class MinioAdmin:
     def user_remove(self, access_key: str) -> str:
         """Delete user"""
         response = self._url_open(
-            "DELETE",
-            _COMMAND.REMOVE_USER,
+            method="DELETE",
+            command=_COMMAND.REMOVE_USER,
             query_params={"accessKey": access_key},
         )
         return response.data.decode()
@@ -374,8 +377,8 @@ class MinioAdmin:
     def user_info(self, access_key: str) -> str:
         """Get information about user"""
         response = self._url_open(
-            "GET",
-            _COMMAND.USER_INFO,
+            method="GET",
+            command=_COMMAND.USER_INFO,
             query_params={"accessKey": access_key},
         )
         return response.data.decode()
@@ -383,7 +386,9 @@ class MinioAdmin:
     def user_list(self) -> str:
         """List all users"""
         response = self._url_open(
-            "GET", _COMMAND.LIST_USERS, preload_content=False,
+            method="GET",
+            command=_COMMAND.LIST_USERS,
+            preload_content=False,
         )
         plain_data = decrypt(
             response, self._provider.retrieve().secret_key,
@@ -398,8 +403,8 @@ class MinioAdmin:
             "isRemove": False
         }).encode()
         response = self._url_open(
-            "PUT",
-            _COMMAND.ADD_UPDATE_REMOVE_GROUP,
+            method="PUT",
+            command=_COMMAND.ADD_UPDATE_REMOVE_GROUP,
             body=body,
         )
         return response.data.decode()
@@ -407,8 +412,8 @@ class MinioAdmin:
     def group_disable(self, group_name: str) -> str:
         """Disable group."""
         response = self._url_open(
-            "PUT",
-            _COMMAND.SET_GROUP_STATUS,
+            method="PUT",
+            command=_COMMAND.SET_GROUP_STATUS,
             query_params={"group": group_name, "status": "disabled"}
         )
         return response.data.decode()
@@ -416,13 +421,17 @@ class MinioAdmin:
     def group_enable(self, group_name: str) -> str:
         """Enable group."""
         response = self._url_open(
-            "PUT",
-            _COMMAND.SET_GROUP_STATUS,
+            method="PUT",
+            command=_COMMAND.SET_GROUP_STATUS,
             query_params={"group": group_name, "status": "enabled"}
         )
         return response.data.decode()
 
-    def group_remove(self, group_name: str, members: str | None = None) -> str:
+    def group_remove(
+            self,
+            group_name: str,
+            members: Optional[str] = None,
+    ) -> str:
         """Remove group or members from a group."""
         data = {
             "group": group_name,
@@ -432,8 +441,8 @@ class MinioAdmin:
             data["members"] = members
 
         response = self._url_open(
-            "PUT",
-            _COMMAND.ADD_UPDATE_REMOVE_GROUP,
+            method="PUT",
+            command=_COMMAND.ADD_UPDATE_REMOVE_GROUP,
             body=json.dumps(data).encode(),
         )
         return response.data.decode()
@@ -441,33 +450,45 @@ class MinioAdmin:
     def group_info(self, group_name: str) -> str:
         """Get group information."""
         response = self._url_open(
-            "GET",
-            _COMMAND.GROUP_INFO,
+            method="GET",
+            command=_COMMAND.GROUP_INFO,
             query_params={"group": group_name},
         )
         return response.data.decode()
 
     def group_list(self) -> str:
         """List groups."""
-        response = self._url_open("GET", _COMMAND.LIST_GROUPS)
+        response = self._url_open(
+            method="GET",
+            command=_COMMAND.LIST_GROUPS,
+        )
         return response.data.decode()
 
-    def policy_add(self, policy_name: str, policy_file: str) -> str:
+    def policy_add(self,
+                   policy_name: str,
+                   policy_file: Optional[str | os.PathLike] = None,
+                   policy: Optional[dict] = None) -> str:
         """Add new policy."""
-        with open(policy_file, encoding='utf-8') as file:
-            response = self._url_open(
-                "PUT",
-                _COMMAND.ADD_CANNED_POLICY,
-                query_params={"name": policy_name},
-                body=file.read().encode(),
-            )
-            return response.data.decode()
+        if not (policy_file is not None) ^ (policy is not None):
+            raise ValueError("either policy_file or policy must be provided")
+        if policy_file:
+            with open(policy_file, encoding='utf-8') as file:
+                body = file.read().encode()
+        else:
+            body = json.dumps(policy).encode()
+        response = self._url_open(
+            method="PUT",
+            command=_COMMAND.ADD_CANNED_POLICY,
+            query_params={"name": policy_name},
+            body=body,
+        )
+        return response.data.decode()
 
     def policy_remove(self, policy_name: str) -> str:
         """Remove policy."""
         response = self._url_open(
-            "DELETE",
-            _COMMAND.REMOVE_CANNED_POLICY,
+            method="DELETE",
+            command=_COMMAND.REMOVE_CANNED_POLICY,
             query_params={"name": policy_name},
         )
         return response.data.decode()
@@ -475,28 +496,31 @@ class MinioAdmin:
     def policy_info(self, policy_name: str) -> str:
         """Get policy information."""
         response = self._url_open(
-            "GET",
-            _COMMAND.CANNED_POLICY_INFO,
+            method="GET",
+            command=_COMMAND.CANNED_POLICY_INFO,
             query_params={"name": policy_name},
         )
         return response.data.decode()
 
     def policy_list(self) -> str:
         """List policies."""
-        response = self._url_open("GET", _COMMAND.LIST_CANNED_POLICIES)
+        response = self._url_open(
+            method="GET",
+            command=_COMMAND.LIST_CANNED_POLICIES,
+        )
         return response.data.decode()
 
     def policy_set(
             self,
             policy_name: str,
-            user: str | None = None,
-            group: str | None = None,
+            user: Optional[str] = None,
+            group: Optional[str] = None,
     ) -> str:
         """Set IAM policy on a user or group."""
         if (user is not None) ^ (group is not None):
             response = self._url_open(
-                "PUT",
-                _COMMAND.SET_USER_OR_GROUP_POLICY,
+                method="PUT",
+                command=_COMMAND.SET_USER_OR_GROUP_POLICY,
                 query_params={"userOrGroup": cast(str, user or group),
                               "isGroup": "true" if group else "false",
                               "policyName": policy_name},
@@ -507,20 +531,20 @@ class MinioAdmin:
     def policy_unset(
             self,
             policy_name: str | list[str],
-            user: str | None = None,
-            group: str | None = None,
+            user: Optional[str] = None,
+            group: Optional[str] = None,
     ) -> str:
         """Unset an IAM policy for a user or group."""
         return self.detach_policy(
             policy_name if isinstance(policy_name, list) else [policy_name],
             user, group)
 
-    def config_get(self, key: str | None = None) -> str:
+    def config_get(self, key: Optional[str] = None) -> str:
         """Get configuration parameters."""
         try:
             response = self._url_open(
-                "GET",
-                _COMMAND.GET_CONFIG,
+                method="GET",
+                command=_COMMAND.GET_CONFIG,
                 query_params={"key": key or "", "subSys": ""},
                 preload_content=False,
             )
@@ -537,7 +561,7 @@ class MinioAdmin:
     def config_set(
             self,
             key: str,
-            config: dict[str, str] | None = None,
+            config: Optional[dict[str, str]] = None,
     ) -> str:
         """Set configuration parameters."""
         data = [key]
@@ -545,20 +569,20 @@ class MinioAdmin:
             data += [f"{name}={value}" for name, value in config.items()]
         body = " ".join(data).encode()
         response = self._url_open(
-            "PUT",
-            _COMMAND.SET_CONFIG,
+            method="PUT",
+            command=_COMMAND.SET_CONFIG,
             body=encrypt(body, self._provider.retrieve().secret_key),
         )
         return response.data.decode()
 
-    def config_reset(self, key: str, name: str | None = None) -> str:
+    def config_reset(self, key: str, name: Optional[str] = None) -> str:
         """Reset configuration parameters."""
         if name:
             key += ":" + name
         body = key.encode()
         response = self._url_open(
-            "DELETE",
-            _COMMAND.DELETE_CONFIG,
+            method="DELETE",
+            command=_COMMAND.DELETE_CONFIG,
             body=encrypt(body, self._provider.retrieve().secret_key),
         )
         return response.data.decode()
@@ -567,8 +591,8 @@ class MinioAdmin:
         """Get historic configuration changes."""
         try:
             response = self._url_open(
-                "GET",
-                _COMMAND.LIST_CONFIG_HISTORY,
+                method="GET",
+                command=_COMMAND.LIST_CONFIG_HISTORY,
                 query_params={"count": "10"},
                 preload_content=False,
             )
@@ -584,8 +608,8 @@ class MinioAdmin:
     def config_restore(self, restore_id: str) -> str:
         """Restore to a specific configuration history."""
         response = self._url_open(
-            "PUT",
-            _COMMAND.RESOTRE_CONFIG_HISTORY,
+            method="PUT",
+            command=_COMMAND.RESOTRE_CONFIG_HISTORY,
             query_params={"restoreId": restore_id}
         )
         return response.data.decode()
@@ -596,8 +620,8 @@ class MinioAdmin:
     ) -> str:
         """Runs a system profile"""
         response = self._url_open(
-            "POST",
-            _COMMAND.START_PROFILE,
+            method="POST",
+            command=_COMMAND.START_PROFILE,
             query_params={"profilerType;": ",".join(profilers)},
         )
         return response.data.decode()
@@ -605,25 +629,25 @@ class MinioAdmin:
     def top_locks(self) -> str:
         """Get a list of the 10 oldest locks on a MinIO cluster."""
         response = self._url_open(
-            "GET",
-            _COMMAND.TOP_LOCKS,
+            method="GET",
+            command=_COMMAND.TOP_LOCKS,
         )
         return response.data.decode()
 
-    def kms_key_create(self, key: str | None = None) -> str:
+    def kms_key_create(self, key: Optional[str] = None) -> str:
         """Create a new KMS master key."""
         response = self._url_open(
-            "POST",
-            _COMMAND.CREATE_KMS_KEY,
+            method="POST",
+            command=_COMMAND.CREATE_KMS_KEY,
             query_params={"key-id": key or ""},
         )
         return response.data.decode()
 
-    def kms_key_status(self, key: str | None = None) -> str:
+    def kms_key_status(self, key: Optional[str] = None) -> str:
         """Get status information of a KMS master key."""
         response = self._url_open(
-            "GET",
-            _COMMAND.GET_KMS_KEY_STATUS,
+            method="GET",
+            command=_COMMAND.GET_KMS_KEY_STATUS,
             query_params={"key-id": key or ""}
         )
         return response.data.decode()
@@ -633,8 +657,8 @@ class MinioAdmin:
         body = json.dumps(
             [peer_site.to_dict() for peer_site in peer_sites]).encode()
         response = self._url_open(
-            "PUT",
-            _COMMAND.SITE_REPLICATION_ADD,
+            method="PUT",
+            command=_COMMAND.SITE_REPLICATION_ADD,
             query_params={"api-version": "1"},
             body=encrypt(body, self._provider.retrieve().secret_key),
         )
@@ -642,7 +666,10 @@ class MinioAdmin:
 
     def get_site_replication_info(self) -> str:
         """Get site replication information."""
-        response = self._url_open("GET", _COMMAND.SITE_REPLICATION_INFO)
+        response = self._url_open(
+            method="GET",
+            command=_COMMAND.SITE_REPLICATION_INFO,
+        )
         return response.data.decode()
 
     def get_site_replication_status(
@@ -651,8 +678,8 @@ class MinioAdmin:
     ) -> str:
         """Get site replication information."""
         response = self._url_open(
-            "GET",
-            _COMMAND.SITE_REPLICATION_STATUS,
+            method="GET",
+            command=_COMMAND.SITE_REPLICATION_STATUS,
             query_params=cast(DictType, options.to_query_params()),
         )
         return response.data.decode()
@@ -661,8 +688,8 @@ class MinioAdmin:
         """Edit site replication with given peer information."""
         body = json.dumps(peer_info.to_dict()).encode()
         response = self._url_open(
-            "PUT",
-            _COMMAND.SITE_REPLICATION_EDIT,
+            method="PUT",
+            command=_COMMAND.SITE_REPLICATION_EDIT,
             query_params={"api-version": "1"},
             body=encrypt(body, self._provider.retrieve().secret_key),
         )
@@ -670,7 +697,7 @@ class MinioAdmin:
 
     def remove_site_replication(
             self,
-            sites: str | None = None,
+            sites: Optional[str] = None,
             all_sites: bool = False,
     ) -> str:
         """Remove given sites or all sites from site replication."""
@@ -683,8 +710,8 @@ class MinioAdmin:
             raise ValueError("either sites or all flag must be given")
         body = json.dumps(data).encode()
         response = self._url_open(
-            "PUT",
-            _COMMAND.SITE_REPLICATION_REMOVE,
+            method="PUT",
+            command=_COMMAND.SITE_REPLICATION_REMOVE,
             query_params={"api-version": "1"},
             body=encrypt(body, self._provider.retrieve().secret_key),
         )
@@ -694,8 +721,8 @@ class MinioAdmin:
         """Set bucket quota configuration."""
         body = json.dumps({"quota": size, "quotatype": "hard"}).encode()
         response = self._url_open(
-            "PUT",
-            _COMMAND.SET_BUCKET_QUOTA,
+            method="PUT",
+            command=_COMMAND.SET_BUCKET_QUOTA,
             query_params={"bucket": bucket},
             body=body
         )
@@ -708,8 +735,8 @@ class MinioAdmin:
     def bucket_quota_get(self, bucket: str) -> str:
         """Get bucket quota configuration."""
         response = self._url_open(
-            "GET",
-            _COMMAND.GET_BUCKET_QUOTA,
+            method="GET",
+            command=_COMMAND.GET_BUCKET_QUOTA,
             query_params={"bucket": bucket}
         )
         return response.data.decode()
@@ -717,16 +744,16 @@ class MinioAdmin:
     def get_data_usage_info(self):
         """Get data usage info"""
         response = self._url_open(
-            "GET",
-            _COMMAND.DATA_USAGE_INFO,
+            method="GET",
+            command=_COMMAND.DATA_USAGE_INFO,
         )
         return response.data.decode()
 
     def get_service_account(self, access_key: str) -> str:
         """Get information about service account"""
         response = self._url_open(
-            "GET",
-            _COMMAND.SERVICE_ACCOUNT_INFO,
+            method="GET",
+            command=_COMMAND.SERVICE_ACCOUNT_INFO,
             query_params={"accessKey": access_key},
             preload_content=False,
         )
@@ -738,8 +765,8 @@ class MinioAdmin:
     def list_service_account(self, user: str) -> str:
         """List service accounts of user"""
         response = self._url_open(
-            "GET",
-            _COMMAND.SERVICE_ACCOUNT_LIST,
+            method="GET",
+            command=_COMMAND.SERVICE_ACCOUNT_LIST,
             query_params={"user": user},
             preload_content=False,
         )
@@ -749,13 +776,15 @@ class MinioAdmin:
         return plain_data.decode()
 
     def add_service_account(self,
-                            access_key: str | None = None,
-                            secret_key: str | None = None,
-                            name: str | None = None,
-                            description: str | None = None,
-                            policy_file: str | None = None,
-                            expiration: str | None = None,
-                            status: str | None = None) -> str:
+                            *,
+                            access_key: Optional[str] = None,
+                            secret_key: Optional[str] = None,
+                            name: Optional[str] = None,
+                            description: Optional[str] = None,
+                            policy: Optional[dict] = None,
+                            policy_file: Optional[str | os.PathLike] = None,
+                            expiration: Optional[str] = None,
+                            status: Optional[str] = None) -> str:
         """
         Add a new service account with the given access key and secret key
         """
@@ -763,7 +792,9 @@ class MinioAdmin:
             raise ValueError("both access key and secret key must be provided")
         if access_key == "" or secret_key == "":
             raise ValueError("access key or secret key must not be empty")
-        data = {
+        if policy_file is not None and policy is not None:
+            raise ValueError("either policy_file or policy must be provided")
+        data: dict[str, Any] = {
             "status": "enabled",
             "accessKey": access_key,
             "secretKey": secret_key,
@@ -775,6 +806,8 @@ class MinioAdmin:
         if policy_file:
             with open(policy_file, encoding="utf-8") as file:
                 data["policy"] = json.load(file)
+        if policy:
+            data["policy"] = policy
         if expiration:
             data["expiration"] = expiration
         if status:
@@ -782,8 +815,8 @@ class MinioAdmin:
 
         body = json.dumps(data).encode()
         response = self._url_open(
-            "PUT",
-            _COMMAND.SERVICE_ACCOUNT_ADD,
+            method="PUT",
+            command=_COMMAND.SERVICE_ACCOUNT_ADD,
             body=encrypt(body, self._provider.retrieve().secret_key),
             preload_content=False,
         )
@@ -793,20 +826,25 @@ class MinioAdmin:
         return plain_data.decode()
 
     def update_service_account(self,
+                               *,
                                access_key: str,
-                               secret_key: str | None = None,
-                               name: str | None = None,
-                               description: str | None = None,
-                               policy_file: str | None = None,
-                               expiration: str | None = None,
-                               status: str | None = None) -> str:
+                               secret_key: Optional[str] = None,
+                               name: Optional[str] = None,
+                               description: Optional[str] = None,
+                               policy_file: Optional[str | os.PathLike] = None,
+                               policy: Optional[dict] = None,
+                               expiration: Optional[str] = None,
+                               status: Optional[str] = None) -> str:
         """Update an existing service account"""
-        args = [secret_key, name, description, policy_file, expiration, status]
+        args = [secret_key, name, description,
+                policy_file, policy, expiration, status]
         if not any(arg for arg in args):
             raise ValueError("at least one of secret_key, name, description, "
-                             "policy_file, expiration or status must be "
-                             "specified")
-        data = {}
+                             "policy_file, policy, expiration or status must "
+                             "be specified")
+        if policy_file is not None and policy is not None:
+            raise ValueError("either policy_file or policy must be provided")
+        data: dict[str, Any] = {}
         if secret_key:
             data["newSecretKey"] = secret_key
         if name:
@@ -816,6 +854,8 @@ class MinioAdmin:
         if policy_file:
             with open(policy_file, encoding="utf-8") as file:
                 data["newPolicy"] = json.load(file)
+        if policy:
+            data["newPolicy"] = policy
         if expiration:
             data["newExpiration"] = expiration
         if status:
@@ -823,8 +863,8 @@ class MinioAdmin:
 
         body = json.dumps(data).encode()
         response = self._url_open(
-            "POST",
-            _COMMAND.SERVICE_ACCOUNT_UPDATE,
+            method="POST",
+            command=_COMMAND.SERVICE_ACCOUNT_UPDATE,
             query_params={"accessKey": access_key},
             body=encrypt(body, self._provider.retrieve().secret_key),
         )
@@ -833,8 +873,8 @@ class MinioAdmin:
     def delete_service_account(self, access_key: str) -> str:
         """Delete a service account"""
         response = self._url_open(
-            "DELETE",
-            _COMMAND.SERVICE_ACCOUNT_DELETE,
+            method="DELETE",
+            command=_COMMAND.SERVICE_ACCOUNT_DELETE,
             query_params={"accessKey": access_key},
         )
         return response.data.decode()
@@ -843,8 +883,8 @@ class MinioAdmin:
             self,
             command: _COMMAND,
             policies: list[str],
-            user: str | None = None,
-            group: str | None = None,
+            user: Optional[str] = None,
+            group: Optional[str] = None,
     ) -> str:
         """Attach or detach policies for builtin or LDAP."""
         if (user is not None) ^ (group is not None):
@@ -854,8 +894,8 @@ class MinioAdmin:
                  key: cast(str, user or group)},
             ).encode()
             response = self._url_open(
-                "POST",
-                command,
+                method="POST",
+                command=command,
                 body=encrypt(body, self._provider.retrieve().secret_key),
                 preload_content=False,
             )
@@ -877,8 +917,8 @@ class MinioAdmin:
     def attach_policy_ldap(
             self,
             policies: list[str],
-            user: str | None = None,
-            group: str | None = None,
+            user: Optional[str] = None,
+            group: Optional[str] = None,
     ) -> str:
         """Attach policies for LDAP."""
         return self._attach_detach_policy(
@@ -888,8 +928,8 @@ class MinioAdmin:
     def detach_policy_ldap(
             self,
             policies: list[str],
-            user: str | None = None,
-            group: str | None = None,
+            user: Optional[str] = None,
+            group: Optional[str] = None,
     ) -> str:
         """Detach policies for LDAP."""
         return self._attach_detach_policy(
@@ -903,7 +943,8 @@ class MinioAdmin:
     ) -> str:
         """List service accounts belonging to the specified user."""
         response = self._url_open(
-            "GET", _COMMAND.IDP_LDAP_LIST_ACCESS_KEYS,
+            method="GET",
+            command=_COMMAND.IDP_LDAP_LIST_ACCESS_KEYS,
             query_params={"userDN": user_dn, "listType": list_type},
             preload_content=False,
         )
@@ -924,7 +965,8 @@ class MinioAdmin:
 
         key, value = ("all", "true") if all_users else ("userDNs", users)
         response = self._url_open(
-            "GET", _COMMAND.IDP_LDAP_LIST_ACCESS_KEYS_BULK,
+            method="GET",
+            command=_COMMAND.IDP_LDAP_LIST_ACCESS_KEYS_BULK,
             query_params={"listType": list_type, key: value},
             preload_content=False,
         )
@@ -936,8 +978,8 @@ class MinioAdmin:
     def attach_policy(
             self,
             policies: list[str],
-            user: str | None = None,
-            group: str | None = None,
+            user: Optional[str] = None,
+            group: Optional[str] = None,
     ) -> str:
         """Attach builtin policies."""
         return self._attach_detach_policy(
@@ -947,8 +989,8 @@ class MinioAdmin:
     def detach_policy(
             self,
             policies: list[str],
-            user: str | None = None,
-            group: str | None = None,
+            user: Optional[str] = None,
+            group: Optional[str] = None,
     ) -> str:
         """Detach builtin policies."""
         return self._attach_detach_policy(
@@ -963,7 +1005,8 @@ class MinioAdmin:
     ) -> str:
         """Get builtin policy entities."""
         response = self._url_open(
-            "GET", _COMMAND.IDP_BUILTIN_POLICY_ENTITIES,
+            method="GET",
+            command=_COMMAND.IDP_BUILTIN_POLICY_ENTITIES,
             query_params={"user": users, "group": groups, "policy": policies},
             preload_content=False,
         )
